@@ -87,15 +87,25 @@ RSpec.describe "bundle executable" do
   end
 
   context "with no arguments" do
-    it "prints a concise help message", bundler: "4" do
-      bundle ""
-      expect(err).to be_empty
+    it "tries to installs by default but print help on missing Gemfile" do
+      bundle "", raise_on_error: false
+      expect(err).to include("Could not locate Gemfile")
+      expect(out).to include("In a future version of Bundler")
+
       expect(out).to include("Bundler version #{Bundler::VERSION}").
         and include("\n\nBundler commands:\n\n").
         and include("\n\n  Primary commands:\n").
         and include("\n\n  Utilities:\n").
         and include("\n\nOptions:\n")
     end
+
+    it "runs bundle install when default_cli_command set to install" do
+      bundle "config set default_cli_command install"
+      bundle "", raise_on_error: false
+      expect(out).to_not include("In a future version of Bundler")
+      expect(err).to include("Could not locate Gemfile")
+      expect(exitstatus).to_not be_zero
+    end
   end
 
   context "when ENV['BUNDLE_GEMFILE'] is set to an empty string" do
@@ -266,10 +276,21 @@ end
 RSpec.describe "bundler executable" do
   it "shows the bundler version just as the `bundle` executable does" do
     bundler "--version"
-    expect(out).to eq("Bundler version #{Bundler::VERSION}")
+    expect(out).to eq(Bundler::VERSION.to_s)
 
-    bundle "config simulate_version 4"
+    bundle "config simulate_version 5"
     bundler "--version"
-    expect(out).to eq("#{Bundler::VERSION} (simulating Bundler 4)")
+    expect(out).to eq("#{Bundler::VERSION} (simulating Bundler 5)")
+  end
+
+  it "shows cli_help when bundler install and no Gemfile is found" do
+    bundler "install", raise_on_error: false
+    expect(err).to include("Could not locate Gemfile")
+
+    expect(out).to include("Bundler version #{Bundler::VERSION}").
+      and include("\n\nBundler commands:\n\n").
+      and include("\n\n  Primary commands:\n").
+      and include("\n\n  Utilities:\n").
+      and include("\n\nOptions:\n")
   end
 end
